﻿using Freebase4net.Operators;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Freebase4net.Tests
{
    [TestClass]
    public class MqlReadServiceAsyncTests
    {
        private MqlReadService _readService;

        public MqlReadServiceAsyncTests()
        {
            _readService = FreebaseServices.CreateMqlReadService();
        }

        [TestMethod]
        public async Task ReadAsync_ObjectWithLikeOperator_StatusCodeOk()
        {
            dynamic film = new ExpandoObject();
            film.id = null;
            film.type = "/film/film";
            film.name = FreebaseHelpers.Operators.CreateLikeOperator("^The *$");

            MqlReadServiceResponse result = await _readService.ReadAsync(film);

            Assert.AreEqual("/film/film", (string)result.Results[0].type);
            Assert.AreEqual(System.Net.HttpStatusCode.OK, result.Status);
        }

        [TestMethod]
        public async Task ReadAsync_DictionaryOfObjects_StatusCodeOk()
        {
            dynamic artist = new ExpandoObject();
            artist.type = "/music/artist";
            artist.name = "The Scorpions";
            artist.album = new Dictionary<Object, Object>();
            artist.album.Add("name", null);
            artist.album.Add("limit", 2);
            artist.album.Add("genre", FreebaseHelpers.CreateArrayWithEmptyObject());


            var result = await _readService.ReadAsync(artist);


            Assert.AreEqual(System.Net.HttpStatusCode.OK, result.Status);
        }



        [TestMethod]
        public async Task ReadAsync_ComplexEmbeddedObject_StatusCodeOk()
        {
            dynamic director = new ExpandoObject();
            director.name = null;
            director.id = null;
            director.type = "/film/editor";
            director.film = new Dictionary<Object, Object>() {
                {"name",null},
                {"id",null},
                {"starring",new Dictionary<Object,Object>()
                    {
                        {"actor", "Theresa Russell"}
                    }
                }
            };


            var result = await _readService.ReadAsync(director);


            Assert.AreEqual(System.Net.HttpStatusCode.OK, result.Status);
        }

        [TestMethod]
        public async Task ReadWithParamsAsync_LimitTo10Items_Returned10Items()
        {
            dynamic artist = new ExpandoObject();
            artist.type = "/music/artist";
            artist.name = FreebaseHelpers.Operators.CreateLikeOperator("^The*$");


            MqlReadServiceResponse result = await _readService.ReadAsync(artist, limit: 10);

            Assert.AreEqual(10, result.Results.Count);
        }

        [TestMethod]
        public void ReadMultipleAsync_MultipleQueriesWithTasksAsResponse_StatusCodeOk()
        {
            dynamic thescorpions = new ExpandoObject();
            thescorpions.name = "The Scorpions";
            thescorpions.type = "/music/artist";

            dynamic thepolice = new ExpandoObject();
            thepolice.name = "The Police";
            thepolice.type = "/music/artist";

            MultipleMqlReadServiceAsyncResponse multiFreebaseResponse = _readService.ReadMultipleAsync(thescorpions, thepolice);
            Task.WaitAll(multiFreebaseResponse.Results.ToArray());

            foreach (var t in multiFreebaseResponse.Results)
            {
                Assert.AreEqual(System.Net.HttpStatusCode.OK, t.Result.Status);
            }
        }

        [TestMethod]
        public void ReadMultipleAsync_MultipleQueriesWithReadyResults_StatusCodeOk()
        {
            dynamic thescorpions = new ExpandoObject();
            thescorpions.name = "The Scorpions";
            thescorpions.type = "/music/artist";

            dynamic thepolice = new ExpandoObject();
            thepolice.name = "The Police";
            thepolice.type = "/music/artist";

            List<MqlReadServiceResponse> multiFreebaseResponse = _readService.ReadMultipleAsyncWithWait(thescorpions, thepolice);

            foreach (var t in multiFreebaseResponse)
            {
                Assert.AreEqual(System.Net.HttpStatusCode.OK, t.Status);
            }
        }

        [TestMethod]
        public async Task ReadAsync_IncludingDescriptionAndImage_StatusCodeOk()
        {
            dynamic artist = new ExpandoObject();
            artist.id = null;
            artist.name = "The Police";
            artist.type = "/music/artist";

            MqlReadServiceResponse result = await _readService.ReadWithDescrAndImageAsync(artist);

            Assert.AreEqual(true, !string.IsNullOrEmpty((string)result.Results[0].Description));
        }

        [TestMethod]
        public async Task ReadWithParamsAsync_WithCursor_ReturnedCursorInResponse()
        {
            dynamic artist = new ExpandoObject();
            artist.type = "/music/artist";
            artist.name = FreebaseHelpers.Operators.CreateLikeOperator("^*$");


            MqlReadServiceResponse result = await _readService.ReadAsync(artist, cursor: new Cursor());


            Assert.AreEqual(true, result.HasCursor);
        }
    }
}
